import os
import sys
import importlib
import functools
import inspect
import asyncio
import nest_asyncio

nest_asyncio.apply() 

def call_func(func, args=None, kwargs=None):
    """
        args:是一个元组
        kwargs:是一个字典
    """
    if args is None:
        args = ()
    if kwargs is None:
        kwargs = {}

    if inspect.isgeneratorfunction(func):
        return list(func(*args, **kwargs))
    elif inspect.isasyncgenfunction(func):
        async def collect_async_gen():
            return [item async for item in func(*args, **kwargs)]
        
        try:
            loop = asyncio.get_running_loop()
        except RuntimeError:
            return asyncio.run(collect_async_gen())
        else:
            async def run_async_gen():
                return await collect_async_gen()
            future = asyncio.ensure_future(run_async_gen())
            while not future.done():
                loop.run_until_complete(asyncio.sleep(0.1))  # 避免阻塞
            return future.result()
    elif inspect.iscoroutinefunction(func):
        coro = func(*args, **kwargs) if inspect.iscoroutinefunction(func) else func
        try:
            loop = asyncio.get_running_loop()
        except RuntimeError:
            return asyncio.run(coro)
        else:
            async def run_coro():
                return await coro
            future = asyncio.ensure_future(run_coro())
            while not future.done():
                loop.run_until_complete(asyncio.sleep(0.1))  # 避免阻塞
            return future.result()
    else:
        return func(*args, **kwargs)
    
class ChangeDirectoryAndPath:
    """Context manager to change the current working directory and sys.path."""

    def __init__(self, module_path):
        self.module_path = module_path
        self.old_path = None

    def __enter__(self):
        self.old_path = os.getcwd()
        sys.path.insert(0, self.module_path)
        os.chdir(self.module_path)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self.module_path in sys.path:
            sys.path.remove(self.module_path)
        os.chdir(self.old_path)
        

class ModuleManager:
    """Context manager for handling module imports and sys.modules state."""

    def __init__(self, module_path):
        self.module_path = module_path
        self.original_modules = sys.modules.copy()

    def __enter__(self):
        """Enter the runtime context related to this object."""
        self.change_dir = ChangeDirectoryAndPath(self.module_path)
        self.change_dir.__enter__()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """Exit the runtime context related to this object."""
        self.change_dir.__exit__(exc_type, exc_value, traceback)
        self.cleanup_modules()

    def cleanup_modules(self):
        """Restore the original sys.modules state."""
        importlib.invalidate_caches()
        for key in list(sys.modules.keys()):
            if key not in self.original_modules:
                del sys.modules[key]
                

def wrapped_func(func, module_path):

    def _wrapped(*args, **kwargs):
        try:
            print(f"调用了 {func.__name__}")
        except:
            print(f"called unknown")
        with ChangeDirectoryAndPath(module_path):
            try:
                result = call_func(func, args, kwargs)
            except Exception as e:
                print(f"调用 {func.__name__} 时发生错误: {str(e)}")
                raise
        
        print(f"{func.__name__} 调用完毕")

        return result
    
    wrapped_function = functools.wraps(func)(_wrapped)
    return wrapped_function




import os
import json


def read_params_file():
    params_file = os.getenv("params_file", None)
    params = None
    if params_file:
        try:
            with open(params_file, "r", encoding="utf-8") as f:
                params = json.load(f)
        except Exception as e:
            print(e)
    return params


params = read_params_file()


def read_json_file(filename):
    json_file = os.path.join(params['appPath'], 'JsonStorage', filename)
    with open(json_file, encoding='utf-8') as f:
        return json.load(f)





import asyncio
import os
import sys
import atexit
import time
import chardet


def better_kill(process, raiseError: bool):
    """优雅地终止进程，如果超时则强制终止"""
    try:
        process.terminate()  # 先尝试优雅终止
        time.sleep(5)  # 等待5秒
    except asyncio.TimeoutError:
        try:
            process.kill()  # 如果超时，强制终止
        except:
            if raiseError:
                raise
            else:
                pass
    except Exception as e:
        # print(f"Error while terminating process: {e}")
        if raiseError:
            raise
        else:
            pass



async def read_stream(stream, log, separator=b'\n'):
    def log_func(data):
        try:
            data = data.decode().strip()
            log(data)
        except:
            try:
                encoding = chardet.detect(data)['encoding']
                data = data.decode(encoding).strip()
            finally:
                log(data)

    while True:
        try:
            line = await stream.readuntil(separator)
            log_func(line)
        except asyncio.IncompleteReadError as e:
            # If the stream ends without a separator, output the remaining content
            if e.partial:
                log_func(e.partial)
            break
        except asyncio.LimitOverrunError as e:
            # If the separator is found but not within the limit, read the chunk
            chunk = await stream.read(e.consumed)
            log_func(chunk)




async def execute_script(script_path, *args, cwd=None, env=None):
    cwd = cwd or os.path.dirname(script_path)
    process = await asyncio.create_subprocess_exec(
        sys.executable,
        "-u",
        script_path,
        *args,
        cwd=cwd,
        env=env,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE,
    )
    return process


async def execute_script_and_block(script_path, log, *args, cwd=None, env=None):
    try:
        # 执行脚本
        process = await execute_script(script_path, *args, cwd=cwd, env=env)
    except Exception as e:
        better_kill(process, False)
        raise Exception(f"Failed to execute script: {e}")

    try:
        # 开始并行读取 stdout 和 stderr
        stdout_task = asyncio.create_task(read_stream(process.stdout, log))
        stderr_task = asyncio.create_task(read_stream(process.stderr, log))

        # 注册atexit处理程序
        async def cleanup():
            better_kill(process, False)
            stdout_task.cancel()
            stderr_task.cancel()
        atexit.register(cleanup)

        # 等待进程完成
        return_code = await process.wait()

        # 等待读取任务完成
        await stdout_task
        await stderr_task

    except Exception as e:
        better_kill(process, False)
        stdout_task.cancel()
        stderr_task.cancel()
        raise Exception(f"Error during script execution or stream reading: {e}")

    # 检查返回码
    if return_code != 0:
        better_kill(process, False)
        raise Exception("Error executing script, return code: {}".format(return_code))

    # 如果一切正常，返回成功
    return "success"



async def execute_script_no_block(script_path, log, *args, cwd=None, env=None):
    process = await execute_script(script_path, *args, cwd=cwd, env=env)

    # Start reading stdout and stderr in parallel
    stdout_task = asyncio.create_task(read_stream(process.stdout, log))
    stderr_task = asyncio.create_task(read_stream(process.stderr, log))

    # Register atexit handler to ensure process cleanup
    def cleanup():
        better_kill(process, False)
        stdout_task.cancel()
        stderr_task.cancel()

    atexit.register(cleanup)

    return process, stdout_task, stderr_task